require "locale";
require "infoImage";

----------------------------------------------
-- Define local functions
----------------------------------------------
local setInfoScrollbarVisibility = function(nLines)
	if nLines > nLinesMax then
		infoScrollbarFrame.Visible = true;
	else
		infoScrollbarFrame.Visible = false;
	end
end

local getTextWidth = function(text)
	local lineWidthMax = 0;
	for textline in string.gfind(text,"([^\n]*)\n") do
		local lineWidth = normalfont:getwidth(textline)
		if lineWidth > lineWidthMax then
			lineWidthMax = lineWidth;
		end
	end
	local textWidth = math.ceil(lineWidthMax);
	return textWidth;
end

local getImageSize = function(imageIndex)
	imageWidth = image[imageIndex]:getwidth();
	imageHeight = image[imageIndex]:getheight();
	if imageWidth > 512 or imageHeight > 512 then
		if imageWidth >= imageHeight then
			imageHeight = imageHeight*512/imageWidth;
			imageWidth = 512;
		else
			imageWidth = imageWidth*512/imageHeight;
			imageHeight = 512;
		end
	end
end

local cleanfilenames = function()
	for n = 1, imageIndexMax do
		imagefilename[n] = nil;
	end
end

local setDiaporamaBoxVisibility = function(imageIndexMax)
	if imageIndexMax > 1 then
		diaporamaBox.Visible = true;
	else
		diaporamaBox.Visible = false;
	end
end

----------------------------------------------
-- Set initial values
----------------------------------------------
infoTextFrameWidth, infoTextFrameHeight = 350, 455;
infoScrollbarFrameW, infoScrollbarFrameH = 9, 425;

cInfo = ctext;
info_first_tick = true;

seltextline = {};
nLines = 0;
nLinesMax = 32;
infoScroll = 0;
infoScrollCoeff = 10;
infoScrollbarBoxH = 0;

selnames = {};
imagefilename = {};
image = {};

imageWidth = 256;
imageHeight = 256;
yImageRatio = 256/imageHeight;
imdx, imdy = 0, 0;
-- Move image up if obsModeBox is disabled.
if not(obsModeBox) then
	imdy = 50;
end

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
infoBox = CXBox:new()
	:init(0, 0, 0, 0)
	--:bordercolor({1, 1, 0, 1})
	--:fillcolor({1,1,0,.2})
	:movable(false)
	:attach(screenBox, width, height, 0, 0);

infoBox.Customdraw =
	function(this)
		textlayout:setfont(normalfont);
		textlayout:setfontcolor(cInfo);
		textlayout:setpos(this.lb+25, this.tb-14);
		textlayout:println(_("More Info"));
	end;

infoCheck = CXBox:new()
		:init(0, 0, 0, 0)
		:bordercolor(cbubordoff)
		--:fillcolor({1,1,0,.2})
		:textfont(normalfont)
		:textcolor(cbutextoff)
		:textpos("center")
		:movetext(0, 9)
		:text("")
		:movable(false)
		:active(true)
		:attach(infoBox, 100, 4, 35, 5)

infoCheck.Customdraw =
	function(this)
		-- Set the initial state of the check-box from 'config.lua'.
		if info_first_tick and enable_info then
			infoCheck.Action();
		end
		info_first_tick = false;

		local sel = celestia:getselection();
		if selection then
			if newselection then
				if infoText[sel:name()] or infoImage[sel:name()..1] then
					enableCheckBox(infoCheck);
					cInfo = ctext;
				else
					disableCheckBox(infoCheck);
					cInfo = cchetextoff;
				end
			end
		else
			disableCheckBox(infoCheck);
			cInfo = cchetextoff;
		end
	end

infoCheck.Action = (function()
	return
		function()
			selname = nil;
			imageIndex = 1;
			infoTextFrame.Visible = not(infoTextFrame.Visible);
			infoImageFrame.Visible = not(infoImageFrame.Visible);
			if infoTextFrame.Visible or infoImageFrame.Visible then
				infoCheck.Text = "x";
			else
				infoCheck.Text = "";
			end
		end
	end) ();

infoTextFrame = CXBox:new()
	:init(0, 0, 0, 0)
	--:bordercolor(cbubordoff)
	--:fillcolor(csetbofill)
	:movable(false)
	:clickable(true)
	:visible(false)
	:attach(screenBox, 0, 30, width-infoTextFrameWidth, height-infoTextFrameHeight-30);

infoTextFrame.Customdraw =
	function(this)
		local sel = celestia:getselection();
		if selection then
			if newselection then
				seltext = infoText[selname];
				if seltext then
					if string.find(seltext, "\n") then
						-- Split the text into lines.
						if string.sub(seltext, -1) ~= "\n" then
							seltext = seltext.."\n";
						end;
						seltext, nLines = string.gsub(seltext, "\n", "\n");
					end
					setInfoScrollbarVisibility(nLines);
					infoTextFrameWidth = getTextWidth(seltext)+30;
					infoTextFrameH = infoTextFrameHeight;
				else
					infoScrollbarFrame.Visible = false;
					infoTextFrameWidth = 0;
					infoTextFrameH = 0;
					selimagefilename = infoImage[sel:name()..1];
					if selimagefilename then
						selimage = celestia:loadtexture(selimagefilename);
					else
						disableCheckBox(infoCheck);
					end
				end
				resetInfoScrollPos = true;
				infoScroll = 0;
			else
				resetInfoScrollPos = false;
			end
			if seltext then
				lineCount = 0;
				-- Display info Text.
				textlayout:setfont(normalfont);
				textlayout:setpos(this.lb+5, this.tb-15);
				textlayout:setlinespacing(normalfont:getheight());
				textlayout:setfontcolor(ctext);
				for seltextline in string.gfind(seltext,"([^\n]*)\n") do
					if lineCount >= infoScroll and lineCount < nLinesMax+infoScroll then
						textlayout:println(seltextline);
					end
					lineCount = lineCount+1;
				end
			end
		else
			disableCheckBox(infoCheck);
			if infoScrollbarFrame.Visible then
				infoScrollbarFrame.Visible = false;
			end
			infoTextFrameWidth = 0;
			infoTextFrameH = 0;
		end
		infoTextFrame:attach(screenBox, 0, 30, width-infoTextFrameWidth, height-infoTextFrameH-30);
	end;

infoScrollbarFrame = CXBox:new()
	--:bordercolor(cbubordoff)
	--:fillcolor({1, 0, 0, 1})
	:visible(true)
	:attach(infoTextFrame, infoTextFrameWidth-infoScrollbarFrameW-3, 10, 3, infoTextFrameHeight-10-infoScrollbarFrameH);

infoScrollbarFrame.Customdraw =
	function(this)
		local lb3,bb3,rb3,tb3 =infoScrollbarFrame:bounds();
		local lb4,bb4,rb4,tb4 = infoScrollbarBox:bounds();
		infoScroll = math_round((tb3-tb4-3)/infoScrollCoeff);
		infoScrollbarFrame:attach(infoTextFrame, infoTextFrameWidth-infoScrollbarFrameW-3, 10, 3, infoTextFrameHeight-10-infoScrollbarFrameH);
	end

infoScrollbarBox = CXBox:new()
	:bordercolor(cbubordoff)
	:fillcolor(cscrollfill)
	:visible(true)
	:movable(true)
	:attach(infoScrollbarFrame, 0, (nLines-nLinesMax)*infoScrollCoeff, 0, 3)

infoScrollbarBox.Customdraw =
	function(this)
		if resetInfoScrollPos then
			if nLines < 100 then
				infoScrollCoeff = 10
			elseif nLines < 1000 then
				infoScrollCoeff = 85*math.log(nLines)/(nLines-nLinesMax)
			else
				infoScrollCoeff = 200/(nLines-nLinesMax)
			end
			infoScrollbarBoxH = infoScrollbarFrameH-6-(nLines-nLinesMax)*infoScrollCoeff;
			infoScrollbarBox:attach(infoScrollbarFrame, 0, infoScrollbarFrameH-infoScrollbarBoxH-3, 0, 3)
		end
	end

infoScrollUpButton = CXBox:new()
	:init(0, 0, 0, 0)
	--:fillcolor(csetbofill)
	--:bordercolor(cbubordoff)
	:textfont(titlefont)
	:textcolor(cclotext)
	--:textpos("center")
	:movetext(-3, -1)
	:text("ˆ")
	:movable(false)
	:attach(infoScrollbarFrame, 0, infoScrollbarFrameH, 0, -11);

infoScrollDownButton = CXBox:new()
	:init(0, 0, 0, 0)
	--:fillcolor(csetbofill)
	--:bordercolor(cbubordoff)
	:textfont(titlefont)
	:textcolor(cclotext)
	--:textpos("center")
	:movetext(-3, -1)
	:text("ˇ")
	:movable(false)
	:attach(infoScrollbarFrame, 0, -11, 0, infoScrollbarFrameH);	-- Inverted but OK (not active).

infoImageFrame = CXBox:new()
	:init(0, 0, 0, 0)
	--:bordercolor(cbubordoff)
	:movable(false)
	:active(false)
	:visible(false)
	:attach(screenBox, width-imageWidth-2, 0, 2, height-imageHeight)

infoImageFrame.Customdraw =
	function(this)
		local sel = celestia:getselection();
		if selection then
			if newselection then
				imageIndex = 1;
				-- Get the number of images for the current selection.
				imageIndexMax = 0;
				for k,v in pairs(infoImage) do
					if infoImage[sel:name()..imageIndexMax+1] then
						imageIndexMax = imageIndexMax + 1
					end
				end
				cleanfilenames();
				setDiaporamaBoxVisibility(imageIndexMax);
				if imageIndexMax > 0 then
					displayimage = true;
				else
					displayimage = false;
				end
			end
		else
			displayimage = false;
		end
		if displayimage then
			selnames[imageIndex] = sel:name()..imageIndex;
			-- Move image out of the toolbar if there's no room.
			if virtualPadBox and padFrame.Visible and not(centerPad) then
				padHeight = padFrameH;
			else
				padHeight = 0;
			end
			if imageHeight > height - toolBoxHeight - padHeight then
					imdx = 150;
				else
					imdx = 0;
			end
			-- Load and display image.
			if imagefilename[imageIndex] ~= infoImage[selnames[imageIndex]] then
				imagefilename[imageIndex] = infoImage[selnames[imageIndex]];
				image[imageIndex] = celestia:loadtexture(imagefilename[imageIndex]);
				getImageSize(imageIndex);
				infoImageFrame:fillimage(image[imageIndex]);
			end
			infoImageFrame:attach(screenBox, width-imageWidth-2-imdx, 0+imdy, 2+imdx, height-imageHeight-imdy);
		else
			if diaporamaBox.Visible then
				diaporamaBox.Visible = false;
			end
			if infoImageFrame.Fillimage then
				infoImageFrame.Fillimage = nil;
				cleanfilenames();
			end
			infoImageFrame:attach(screenBox, width, 0, 0, height);
		end
	end;

diaporamaBox = CXBox:new()
	:init(0,0,0,0)
	--:bordercolor({1, 0 , 0, 1})
	:movable(false)
	:active(false)
	:visible(true)
	:attach(infoImageFrame, imageWidth-60, imageHeight-16, 2, 2);

diaporamaBox.Customdraw =
	function(this)
		-- Display image index in the case of a diaporama.
		textlayout:setfont(smallfont);
		textlayout:setfontcolor(cbutextoff);
		textlayout:setpos(this.lb+19, this.tb-10);
		textlayout:println(imageIndex.." / "..imageIndexMax);
	end;

nextButton = CXBox:new()
	:init(0,0,0,0)
	--:bordercolor(cbubordoff)
	:fillcolor(cslidebofill)
	:textfont(smallfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 6)
	:text(">")
	:attach(diaporamaBox, 41, 0, 0, 0)
	:movable(false)
	:visible(true)
	:active(true)

nextButton.Action = (function()
	return
		function()
			imageIndex = imageIndex + 1;
			cleanfilenames();
			selnames[imageIndex] = selname..imageIndex;
			if not(infoImage[selnames[imageIndex]]) then
				imageIndex = 1;
			end
		end
	end) ();

previousButton = CXBox:new()
	:init(0,0,0,0)
	--:bordercolor(cbubordoff)
	:fillcolor(cslidebofill)
	:textfont(smallfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 6)
	:text("<")
	:attach(diaporamaBox, 0, 0, 41, 0)
	:movable(false)
	:visible(true)
	:active(true)

previousButton.Action = (function()
	return
		function()
			imageIndex = imageIndex - 1;
			cleanfilenames();
			selnames[imageIndex] = selname..imageIndex;
			if not(infoImage[selnames[imageIndex]]) then
				imageIndex = imageIndexMax;
			end
		end
	end) ();