require "locale";

----------------------------------------------
-- Define some useful global values
----------------------------------------------
KM_PER_LY = 9460730472580.8;
KM_PER_AU = 149597870.7;
MLY_PER_KM = 1e6/KM_PER_LY;
degToRad = math.pi/180;

----------------------------------------------
-- Define some useful global functions
----------------------------------------------
format = {};

format.km =
	function(this,km)
		local sign, value, units
		if not (type(km) == "number") then return km end;
		if km < 0 then sign = -1 else sign = 1 end;
		km = math.abs(km);
		if km > 1000000000000 then
			value = km/KM_PER_LY
			units = _("ly");
		elseif km >= 100000000 then 
			value = km/KM_PER_AU;
			units = _("AU");
		else
			value = km
			units = _("km");
		end;
		return string.format("%.2f",sign*value).." "..units;
	end

format.mly =
	function(this,mly)
		if  not (type(mly) == "number")  then
			return "?" ;
		end;
		return format:km(mly*KM_PER_LY/1e6);
	end

format.pos =
	function(this,p)
		if not p then return "nil pos" end;
		return "[ " .. format:mly(p.x) .. "  ,  "..format:mly(p.y) .."  ,  ".. format:mly(p.z).." ]";
	end 

format.degdms =
	function(this,deg)
		local	a = math.abs(deg);
  		local	d = math.floor(a);
		local	r = (a-d)*60;
		local	m = math.floor(r);
		local	s = (r-m)*60;
		if deg < 0 then d = -d end;
		--return string.format("%0.0f %02.0f' %04.2f'' (%0.3f)",d,m,s,deg);
		return string.format("%0.0f° %02.0f' %2.0f''",d,m,s);
	end

format.deghms =
	function(this,deg)
		local	a = math.abs(deg/15);
  		local	d = math.floor(a);
		local	r = (a-d)*60;
		local	m = math.floor(r);
		local	s = (r-m)*60;
		--if deg < 0 then d = -d end;
		return string.format("%0.0fh %02.0fm %2.0fs",d,m,s);
	end

format.raddms =
	function(this,rad)
	local deg = rad/degToRad;
	return format:degdms(deg);
	end

format.fracpercent =
	function(this,f)
	return string.format("%0.2f%%",f*100);
	end

format.twodigits =
	function(this,f)
	return string.format("%02.0f",f);
	end

math_round = function(d)
	return math.floor(d + 0.5);
end;

math_sign = function(d)
	if d == 0 then
		return 0;
	else
		return d/math.abs(d);
	end
end;